/**
 * gclc.system:net.bigeon.gclc.system.ExecSystemCommand.java
 * Created on: Jun 20, 2016
 */
package net.bigeon.gclc.system;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;
import fr.bigeon.gclc.manager.ConsoleManager;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class ExecSystemCommand extends Command {

    /** The command default name */
    private static final String COMMAND_DEFAULT_NAME = "exec"; //$NON-NLS-1$
    /** The end of line separator */
    private static final String EOL = System.lineSeparator();
    /** The class logger */
    private static final Logger LOGGER = Logger
            .getLogger(ExecSystemCommand.class.getName());
    /** The manager for the application's user interface */
    private final ConsoleManager manager;

    /** @param name the name of the command (the input from the manager that
     *            should trigger this command)
     * @param manager the console manager for input and outputs */
    public ExecSystemCommand(String name, ConsoleManager manager) {
        super(name);
        this.manager = manager;
    }

    /** @param manager the console manager for input and outputs */
    public ExecSystemCommand(ConsoleManager manager) {
        super(COMMAND_DEFAULT_NAME);
        this.manager = manager;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#execute(java.lang.String[]) */
    @SuppressWarnings("resource")
    @Override
    public void execute(String... args) throws CommandRunException {
        Process proc;
        try {
            proc = Runtime.getRuntime().exec(args);
        } catch (IOException e2) {
            LOGGER.log(Level.SEVERE, "Unable to run process", e2); //$NON-NLS-1$
            return;
        }

        final InputStream is = proc
                .getInputStream();
        Thread th = new Thread(new Runnable() {

            @SuppressWarnings("synthetic-access")
            @Override
            public void run() {
                try {
                    readToEnd(is);
                    is.close();
                } catch (CommandRunException e) {
                    LOGGER.log(Level.WARNING,
                            "Manager was closed in the meantime...", e); //$NON-NLS-1$
                } catch (IOException e) {
                    LOGGER.log(Level.WARNING, "Input stream was closed...", e); //$NON-NLS-1$
                }
            }
        });
        th.start();
        manager.setPrompt("");
        final OutputStream os = proc.getOutputStream();
        try (BufferedWriter writer = new BufferedWriter(
                new OutputStreamWriter(os))) {
            while (th.isAlive()) {
                String user;
                try {
                    user = manager.prompt();
                } catch (IOException e) {
                    throw new CommandRunException(
                            CommandRunExceptionType.INTERACTION,
                            "manager was closed", e, this); //$NON-NLS-1$
                }
                writer.write(user + EOL);
            }
        } catch (IOException e1) {
            throw new CommandRunException(CommandRunExceptionType.INTERACTION,
                    "manager was closed", e1, this); //$NON-NLS-1$
        }
    }

    /** @param is the input stream
     * @throws CommandRunException if the manager was closed while writing the
     *             stream */
    protected void readToEnd(InputStream is) throws CommandRunException {
        int c;
        try {
            while ((c = is.read()) != -1) {
                try {
                    manager.print(Character.valueOf((char) c).toString());
                } catch (IOException e) {
                    throw new CommandRunException(
                            CommandRunExceptionType.INTERACTION,
                            "manager was closed", e, this); //$NON-NLS-1$
                }
            }
        } catch (IOException e) {
            LOGGER.log(Level.INFO, "input stream reading failed", e); //$NON-NLS-1$
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        return "  The system command is a system dependend command like sh on linux or" + //$NON-NLS-1$
               System.lineSeparator() + "powershell on windows." + //$NON-NLS-1$
               System.lineSeparator() + System.lineSeparator() +
               "  As an example if you give \"cat /etc/hostname\" as argument, on a linux" + //$NON-NLS-1$
               System.lineSeparator() +
               "system, you would get the computer name." + //$NON-NLS-1$
               System.lineSeparator();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usagePattern() */
    @Override
    protected String usagePattern() {
        return "  CMD <system command>"; //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        return "Execute a system command"; //$NON-NLS-1$
    }

}
