/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.ParametrizedCommand.java
 * Created on: Dec 24, 2014
 */
package fr.bigeon.gclc.command;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;
import fr.bigeon.gclc.exception.InvalidParameterException;
import fr.bigeon.gclc.manager.ConsoleManager;

/** <p>
 * A command relying on the {@link CommandParameters} to store parameters values
 *
 * @author Emmanuel BIGEON */
public abstract class ParametrizedCommand extends Command {

    /** If the command may use interactive prompting for required parameters
     * that
     * were not provided on execution */
    private boolean interactive = true;
    /** The manager */
    protected final ConsoleManager manager;
    /** The boolean parameters mandatory status */
    private final Set<String> boolParams = new HashSet<>();
    /** The string parameters mandatory status */
    private final Map<String, Boolean> stringParams = new HashMap<>();
    /** The parameters mandatory status */
    private final Map<String, Boolean> params = new HashMap<>();
    /** The restriction of provided parameters on execution to declared
     * paramters
     * in the status maps. */
    private final boolean strict;

    /** @param manager the manager
     * @param name the name */
    public ParametrizedCommand(ConsoleManager manager, String name) {
        this(manager, name, true);
    }

    /** @param name the name */
    public ParametrizedCommand(String name) {
        this(null, name, true);
    }

    /** @param manager the manager
     * @param name the name
     * @param strict if the arguments are restricted to the declared ones */
    public ParametrizedCommand(ConsoleManager manager, String name,
            boolean strict) {
        super(name);
        this.manager = manager;
        interactive = manager != null;
        this.strict = strict;
    }

    /** @param name the name
     * @param strict if the arguments are restricted to the declared ones */
    public ParametrizedCommand(String name, boolean strict) {
        this(null, name, strict);
    }

    /** <p>
     * Add a parameter to the defined parameters
     *
     * @param param the parameter identification
     * @param stringParameter if the parameter is a parameter with an argument
     * @param needed if the parameter is required
     * @throws InvalidParameterException if the parameter was invalid */
    protected void addParameter(String param, boolean stringParameter,
                                boolean needed) throws InvalidParameterException {
        if (params.containsKey(param)) {
            checkParam(param, stringParameter, needed);
            return;
        }
        if (stringParameter) {
            stringParams.put(param, Boolean.valueOf(needed));
            params.put(param, Boolean.valueOf(needed));
        } else {
            if (needed) {
                // ERROR the boolean parameters cannot be needed
                throw new InvalidParameterException(
                        "Boolean parameter are present by their very nature. They should not be defined as needed"); //$NON-NLS-1$
            }
            boolParams.add(param);
            params.put(param, Boolean.valueOf(false));
        }
    }

    /** @param param the parameter
     * @param stringParameter the string parameter type
     * @param needed if the parameter is needed
     * @throws InvalidParameterException if the new definition is invalid */
    private void checkParam(String param, boolean stringParameter,
                            boolean needed) throws InvalidParameterException {
        if (stringParameter) {
            if (stringParams.containsKey(param)) {
                Boolean need = Boolean.valueOf(
                        needed || stringParams.get(param).booleanValue());
                stringParams.put(param, need);
                params.put(param, need);
                return;
            }
            throw new InvalidParameterException(
                    "Parameter is already defined as boolean"); //$NON-NLS-1$
        }
        if (stringParams.containsKey(param) || needed) {
            throw new InvalidParameterException(
                    "Parameter is already defined as string"); //$NON-NLS-1$
        }
    }

    /** @param parameters the command parameters */
    protected abstract void doExecute(CommandParameters parameters);

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#execute(java.lang.String[]) */
    @SuppressWarnings("boxing")
    @Override
    public final void execute(String... args) throws CommandRunException {
        final CommandParameters parameters = new CommandParameters(
                boolParams, stringParams.keySet(), strict);
        if (!parameters.parseArgs(args)) {
            // the parameters could not be correctly parsed
            throw new CommandRunException(CommandRunExceptionType.USAGE,
                    "Unable to read arguments", this); //$NON-NLS-1$
        }
        final List<String> toProvide = new ArrayList<>();
        for (final Entry<String, Boolean> string : params.entrySet()) {
            if (string.getValue() && parameters.get(string.getKey()) == null) {
                if (!interactive) {
                    throw new CommandRunException(
                            CommandRunExceptionType.INTERACTION,
                            "Missing required parameter " + string.getKey(), //$NON-NLS-1$
                            this);
                }
                toProvide.add(string.getKey());
            }
        }
        // for each needed parameters that is missing, prompt the user.
        fillParameters(toProvide, parameters);
        doExecute(parameters);
    }

    /** @param parameters the parameter list to complete
     * @param toProvide the parameters to ask for
     * @throws CommandRunException if the manager was closed */
    private void fillParameters(List<String> toProvide,
                                CommandParameters parameters) throws CommandRunException {
        for (final String string : toProvide) {
            String value;
            try {
                value = manager.prompt(string);
                while (value.isEmpty()) {
                    value = manager.prompt(string);
                }
            } catch (IOException e) {
                throw new CommandRunException(
                        CommandRunExceptionType.INTERACTION,
                        "Interactive command but manager closed...", e, this); //$NON-NLS-1$
            }
            parameters.set(string, value);
        }
    }

    /** @return the set of boolean parameters */
    public Set<String> getBooleanParameters() {
        return Collections.unmodifiableSet(boolParams);
    }

    /** @return the stringParams */
    public Set<String> getStringParameters() {
        return stringParams.keySet();
    }

    /** @return the stringParams */
    public Set<String> getParameters() {
        return params.keySet();
    }

    /** @param param the parameter name
     * @return if the parameter is needed */
    public boolean isNeeded(String param) {
        return params.containsKey(param) && params.get(param).booleanValue();
    }

    /** @return the strict */
    public boolean isStrict() {
        return strict;
    }

    /** @return the interactive */
    public boolean isInteractive() {
        return interactive;
    }
}
